#!/usr/bin/env python
import os
import shutil
import radiomics
from radmxtk.pyrads_gui.controller import Controller

if __name__ == '__main__':

	testCases = ['lung1','lung2']

	exampleData_folder = os.path.abspath(os.path.join(os.getcwd(),'exampleData'))

	dir_valid = True
	if os.path.isdir(exampleData_folder):
		for testCase in testCases:
			testCase_folder = os.path.join(exampleData_folder,testCase)
			if os.path.isdir(testCase_folder):
				imageName = os.path.join(testCase_folder,'CT.nrrd')
				if os.path.isfile(imageName) == True:
					roiName = os.path.join(testCase_folder,'tumour.nrrd')
					if os.path.isfile(roiName) == False:
						dir_valid = False
						shutil.rmtree(exampleData_folder)
						break
				else:
					dir_valid = False
					shutil.rmtree(exampleData_folder)
					break
			else:
				dir_valid = False
				shutil.rmtree(exampleData_folder)
				break
	else:
		dir_valid = False

	if dir_valid == False:
		os.mkdir(exampleData_folder)

		for testCase in testCases:
			testCase_folder = os.path.join(exampleData_folder,testCase)
			os.mkdir(testCase_folder)

			imageName, roiName = radiomics.getTestCase(testCase, testCase_folder)
			_, image_ext = os.path.splitext(imageName)
			_, roi_ext = os.path.splitext(roiName)
			new_imageName = os.path.join(testCase_folder,'CT'+image_ext)
			new_roiName = os.path.join(testCase_folder,'tumour'+roi_ext)

			os.rename(imageName,new_imageName)
			os.rename(roiName,new_roiName)
			
	c = Controller(exampleData_folder)


import tkinter as tk
from tkinter import messagebox

from . import models
from . import views

class Controller():
	def __init__(self, initial_data_dir):
		self.step_name = 'Step 1'
		self.directory_radio_flag = False
		self.directory_browse_flag = False
		self.file_list_modified = False

		self.root = tk.Tk()
		self.model = models.Model()
		if initial_data_dir is not None:
			self.model.default_data_path = initial_data_dir
		self.view = views.View(self.root, self)

		self.root.title("PyRadiomics Utility")
		self.root['bg'] = '#F0F0F0'

		width=812; height=557
		screenwidth = self.root.winfo_screenwidth(); screenheight = self.root.winfo_screenheight()

		alignstr = '%dx%d+%d+%d' % (width, height, (screenwidth - width) / 2, (screenheight - height) / 2)
		self.root.geometry(alignstr)
		self.root.resizable(width=False, height=False)
		self.root.deiconify()
		
		self.root.mainloop()

	def next(self):
		next_step = {'Step 1':'Step 2','Step 2':'Step 3','Step 3':'Step 4','Step 4':'Step 5','Step 5':'ready','ready':'extraction','extraction':'close'}[self.step_name]

		if next_step == 'Step 2':
			self.step_name = 'Step 2'
			self.view.show_step2(self.root,self)

			self.view.path_entry.contents.set(self.model.data_path)
			self.view.wiz_steps.deactivate('Step 1')
			self.view.wiz_steps.activate('Step 2')

			if len(self.model.data_path) == 0:
				self.view.next_button['state'] = 'disabled'
			else:
				self.view.next_button['state'] = 'normal'

		elif next_step == 'Step 3':	

			if self.model.directory_setup == 'same': # skip to file verification
				self.step_name = 'Step 5'
				self.model.image_path = self.model.data_path + '/' + self.model.subject_list[0] 
				self.model.roi_path = self.model.data_path + '/' + self.model.subject_list[0]

				if self.file_list_modified == False:
					self.model.init_image_files_list()
					self.model.init_roi_files_list()
					self.model.init_all_files_list()
					self.update_file_lists()

				self.view.show_step5(self.root,self)
				
				self.view.wiz_steps.deactivate('Step 2')
				self.view.wiz_steps.deactivate('Step 3')
				self.view.wiz_steps.deactivate('Step 4')
				self.view.wiz_steps.activate('Step 5')

				if self.model.check_file_list_valid():
					self.view.next_button['state'] = 'normal'
				else:
					self.view.next_button['state'] = 'disabled'

			elif self.model.directory_setup == 'diff': 
				self.step_name = 'Step 3'
				self.view.show_step3(self.root,self)

				self.view.path_entry.contents.set(self.model.image_path)
				self.view.wiz_steps.deactivate('Step 2')
				self.view.wiz_steps.activate('Step 3')

				if self.file_list_modified == False:
					self.model.init_image_files_list()
					self.model.init_roi_files_list()
					self.model.init_all_files_list()
					self.update_file_lists()

				if len(self.model.image_path) == 0:
					self.view.next_button['state'] = 'disabled'
				else:
					self.view.next_button['state'] = 'normal'

		elif next_step == 'Step 4':
			self.step_name = 'Step 4'
			self.view.show_step4(self.root,self)

			self.view.path_entry.contents.set(self.model.roi_path)
			self.view.wiz_steps.deactivate('Step 3')
			self.view.wiz_steps.activate('Step 4')

			self.view.next_button['state'] = 'disabled'

			if len(self.model.roi_path) == 0:
				self.view.next_button['state'] = 'disabled'
			else:
				self.view.next_button['state'] = 'normal'

			if self.file_list_modified == False:
				self.model.init_image_files_list()
				self.model.init_roi_files_list()
				self.model.init_all_files_list()
				self.update_file_lists()

		elif next_step == 'Step 5':			
			self.step_name = 'Step 5'
			self.view.show_step5(self.root,self)

			self.view.wiz_steps.activate('Step 5')
			self.view.wiz_steps.deactivate('Step 4')

			if self.model.check_file_list_valid():
				self.view.next_button['state'] = 'normal'
			else:
				self.view.next_button['state'] = 'disabled'

		elif next_step == 'ready':
			self.step_name = 'ready'
			self.view.show_extraction(self.root,self)
			self.view.wiz_steps.deactivate('Step 5')

			ready_flag, missing_file = self.model.validate_directory()
			if ready_flag:
				self.model.init_output_file()
			else:
				messagebox.showerror('Missing File(s)', 'Could not locate\n '+missing_file)
				self.view.next_button['state'] = 'disabled'

		elif next_step == 'extraction':
			self.step_name = 'extraction'
			self.view.instruction['text'] = 'Running PyRadiomics...'
			self.view.next_button['state'] = 'disabled'
			self.view.back_button['state'] = 'disabled'
			self.root.update()
			self.model.extract_features()
			self.view.instruction['text'] = 'Complete!'
			self.view.next_button['text'] = 'Close'
			self.view.show_save_load_label()
			self.view.save_load_label['text'] = 'Features saved in:\n    '+self.model.output_file
			self.view.next_button['state'] = 'normal'
			self.view.back_button['state'] = 'normal'
		elif next_step == 'close':
			answer = messagebox.askokcancel("Ready to Close","Are you sure you want to quit?")
			if answer:
				self.root.destroy()

	def back(self):
		back_step = {'Step 1':None,'Step 2':'Step 1','Step 3':'Step 2','Step 4':'Step 3','Step 5':'Step 4','ready':'Step 5','extraction':'ready'}[self.step_name]

		if back_step == 'Step 1':
			self.view.show_step1(self.root,self)
			self.load_options()
			self.view.next_button['state'] = 'normal'
			self.view.wiz_steps.deactivate('Step 2')
			self.view.wiz_steps.activate('Step 1')
			self.step_name = 'Step 1'
			self.view.next_button['state'] = 'normal'

		elif back_step == 'Step 2':
			self.view.show_step2(self.root,self)
			self.view.path_entry.contents.set(self.model.data_path)
			self.view.next_button['state'] = 'normal'
			self.view.wiz_steps.deactivate('Step 3')
			self.view.wiz_steps.activate('Step 2')
			self.step_name = 'Step 2'
		
		elif back_step == 'Step 3':
			self.view.show_step3(self.root,self)
			self.view.path_entry.contents.set(self.model.image_path)
			self.view.next_button['state'] = 'normal'
			self.view.wiz_steps.deactivate('Step 4')
			self.view.wiz_steps.activate('Step 3')
			self.step_name = 'Step 3'

		elif back_step == 'Step 4':
			if self.model.directory_setup == 'same':
				self.view.show_step2(self.root,self)
				self.view.path_entry.contents.set(self.model.data_path)
				self.view.next_button['state'] = 'normal'
				self.view.wiz_steps.deactivate('Step 5')
				self.view.wiz_steps.activate('Step 2')
				self.step_name = 'Step 2'
			else:
				self.view.show_step4(self.root,self)
				self.view.path_entry.contents.set(self.model.roi_path)
				self.view.next_button['state'] = 'normal'
				self.view.wiz_steps.deactivate('Step 5')
				self.view.wiz_steps.activate('Step 4')
				self.step_name = 'Step 4'

		elif back_step == 'Step 5':
			self.view.show_step5(self.root,self)

			self.view.next_button['state'] = 'normal'
			self.view.next_button['text'] = 'Next'

			self.view.wiz_steps.activate('Step 5')
			self.step_name = 'Step 5'

		elif back_step == 'ready':
			self.view.hide_save_load_label()
			self.root.update()
			self.view.show_extraction(self.root,self)
			self.view.instruction['text'] = 'Proceed to Extraction:'
			self.step_name = 'ready'

	def load(self):
		self.model.load()
		if len(self.model.yaml_file) > 0:
			self.view.save_load_label_text = 'Parameters loaded from:\n  '+self.model.yaml_file 
			self.view.show_save_load_label()
			self.view.next_button['state'] = 'normal'
			self.view.next_button_state = 'normal'
			self.load_options()

	def load_options(self):
		options_objects = [self.view.setting,self.view.voxelSetting,self.view.featureClass]
		for options_object in options_objects:
			for key,val in options_object.options_vars.items():
				options_object.options_vars[key].set(0)
				options_object.hide_entries(key)
		
		if 'voxelSetting' not in list(self.model.yaml_dicts.keys()):
			self.model.yaml_dicts['voxelSetting'] = {}
			self.model.yaml_dicts['voxelSetting']['voxelBased'] = False
		else:
			if len(self.model.yaml_dicts['voxelSetting']) == 0:
				self.model.yaml_dicts['voxelSetting'] = {}
			self.model.yaml_dicts['voxelSetting']['voxelBased'] = True

		for yaml_key,yaml_val in self.model.yaml_dicts.items():
			if yaml_key == 'imageType':
				pass
			else:
				yaml_dict = yaml_val
				if yaml_key == 'setting':
					for key,val in yaml_dict.items():

						if key in self.view.setting.options_vars.keys():
							if isinstance(val, bool):
								self.view.setting.options_vars[key].set(val)

							elif type(val) == type(float()) or type(val) == type(int()): 
								self.view.setting.options_vars[key].set(1)
								self.view.setting.options_strings[key].set(val) 
								self.view.setting.show_entries(key)

							elif type(val) == type(list()):
								self.view.setting.options_vars[key].set(1)
								for i in range(3):
									self.view.setting.options_strings[key][i].set(val[i]) 
									self.view.setting.show_entries(key)
								
				elif yaml_key == 'voxelSetting':
					if yaml_dict['voxelBased'] == True:
						self.view.featureClass.options_buttons['shape']['state'] = 'disabled'
						for option in self.view.voxelSetting.options_list:
							self.view.voxelSetting.options_buttons[option]['state'] = 'normal'
						for key,val in yaml_dict.items():
							if isinstance(val, bool):
								self.view.voxelSetting.options_vars[key].set(val)

							elif type(val) == type(float()) or type(val) == type(int()) or val == 'nan': 
								self.view.voxelSetting.options_vars[key].set(1)
								self.view.voxelSetting.options_strings[key].set(val) 
								self.view.voxelSetting.show_entries(key)

							elif type(val) == type(list()):
								self.view.voxelSetting.options_vars[key].set(1)
								for i in range(3):
									self.view.voxelSetting.options_strings[key][i].set(val[i]) 
									self.view.voxelSetting.show_entries(key)

					elif yaml_dict['voxelBased'] == False:
						self.view.voxelSetting.deactivate_all()
						self.view.voxelSetting.options_vars['voxelBased'].set(0)
						self.view.voxelSetting.options_buttons['voxelBased']['state'] = 'normal'

				elif yaml_key == 'featureClass':
					for key,val in yaml_dict.items():
						self.view.featureClass.options_vars[key].set(1)
						if key == 'shape' and val == True:
							self.view.voxelSetting.deactivate_all()
							self.view.voxelSetting.options_vars['voxelBased'].set(0)

	def save(self):
		setting_dict = {}
		for i, option in enumerate(self.view.setting.options_list):
			if self.view.setting.options_vars[option].get() == 1:
				if self.view.setting.options_dicts[i]['has_entry']:
					setting_dict[option] = float(self.view.setting.options_strings[option].get())
				elif self.view.setting.options_dicts[i]['has_entries']:
					x = float(self.view.setting.options_strings[option][0].get())
					y = float(self.view.setting.options_strings[option][1].get())
					z = float(self.view.setting.options_strings[option][2].get())
					setting_dict[option] = [x,y,z]
				else:
					setting_dict[option] = True

		voxelSetting_dict = {}
		for i, option in enumerate(self.view.voxelSetting.options_list):
			if self.view.voxelSetting.options_vars[option].get() == 1:
				if self.view.voxelSetting.options_dicts[i]['has_entry']:
					entry_string = self.view.voxelSetting.options_strings[option].get()

					if type(self.view.voxelSetting.params['option_list'][i][1]['entry_type']) == type(int(0)):
						voxelSetting_dict[option] = int(entry_string)
					else:
						voxelSetting_dict[option] = float(entry_string)

				elif self.view.voxelSetting.options_dicts[i]['has_entries']:
					x = float(self.view.voxelSetting.options_strings[option][0].get())
					y = float(self.view.voxelSetting.options_strings[option][1].get())
					z = float(self.view.voxelSetting.options_strings[option][2].get())
					voxelSetting_dict[option] = [x,y,z]
				else:
					voxelSetting_dict[option] = True

		featureClass_dict = {}
		for i, option in enumerate(self.view.featureClass.options_list):
			if self.view.featureClass.options_vars[option].get() == 1:
				featureClass_dict[option] = True

		self.model.setting_dict = setting_dict
		self.model.voxelSetting_dict = voxelSetting_dict
		self.model.featureClass_dict = featureClass_dict

		self.model.save(setting_dict,voxelSetting_dict,featureClass_dict)

		if len(self.model.yaml_file)>0:
			self.view.save_load_label_text = 'Parameters saved to:\n  '+self.model.yaml_file 
			self.view.show_save_load_label() 
			self.view.next_button['state'] = 'normal'
			self.view.next_button_state = 'normal'
			self.load_options()

	def browse_dir(self):
		dir_path = self.model.browse_dir(self.step_name)
		self.view.path_entry.contents.set(dir_path)
		self.view.path_entry.focus_set()

		if self.step_name == 'Step 2' and len(self.model.subject_list) > 0:
			self.directory_browse_flag = True
			self.file_list_modified = False
			if self.directory_radio_flag == True:
				self.view.next_button['state'] = 'normal'
			self.model.default_image_path = self.model.data_path + '/' + self.model.subject_list[0]
			self.model.default_roi_path = self.model.data_path + '/' + self.model.subject_list[0]

		elif self.step_name == 'Step 3' and len(self.model.image_path) > 0:
			self.model.init_image_files_list()
			if len(self.model.roi_files_list) > 0:
				self.model.init_all_files_list()
			self.update_file_lists()
			self.view.next_button['state'] = 'normal'

		elif self.step_name == 'Step 4' and len(self.model.roi_path) > 0:
			self.view.next_button['state'] = 'normal'
			self.model.init_image_files_list()
			self.model.init_roi_files_list()
			self.model.init_all_files_list()
			self.update_file_lists()

	def image_add_button(self):
		self.file_list_modified = True
		indices = self.view.all_files_list.listbox.curselection()
		if len(indices) > 0:
			index_to_add = indices[0]
			file_to_add = self.model.all_files_list[index_to_add]

			lines = self.view.image_files_list.listbox.size()
			if self.model.image_files_list.count(file_to_add) == 0:
				self.view.image_files_list.listbox.insert(lines, file_to_add)
				self.model.image_files_list.append(file_to_add)

			if self.model.check_file_list_valid():
				self.view.next_button['state'] = 'normal'
			else:
				self.view.next_button['state'] = 'disabled'

	def roi_add_button(self):
		self.file_list_modified = True
		indices = self.view.all_files_list.listbox.curselection()
		if len(indices) > 0:
			index_to_add = indices[0]
			file_to_add = self.model.all_files_list[index_to_add]

			lines = self.view.roi_files_list.listbox.size()
			if self.model.roi_files_list.count(file_to_add) == 0:
				self.view.roi_files_list.listbox.insert(lines, file_to_add)
				self.model.roi_files_list.append(file_to_add)

			if self.model.check_file_list_valid():
				self.view.next_button['state'] = 'normal'
			else:
				self.view.next_button['state'] = 'disabled'

	def update_file_lists(self):
		self.view.params['all_files_list']['height'] = len(self.model.all_files_list) * (12+10)
		self.view.params['image_files_list']['height'] = self.view.params['all_files_list']['height']
		self.view.params['image_files_list']['button_y'] = \
			self.view.params['image_files_list']['height'] + self.view.params['image_files_list']['y'] + 4
		self.view.params['roi_files_list']['height'] = self.view.params['all_files_list']['height']
		self.view.params['roi_files_list']['button_y'] = \
			self.view.params['roi_files_list']['height'] + self.view.params['roi_files_list']['y'] + 4

		self.view.all_files_list.update_file_list(self.model.all_files_list)
		self.view.image_files_list.update_file_list(self.model.image_files_list)
		self.view.roi_files_list.update_file_list(self.model.roi_files_list)

	def all_files_list_event(self, event):
		all_files_cursor = self.view.all_files_list.listbox.curselection()
		image_files_cursor = self.view.image_files_list.listbox.curselection()
		roi_files_cursor = self.view.roi_files_list.listbox.curselection()

		if len(all_files_cursor) > 0 or len(image_files_cursor) == 0 or len(roi_files_cursor) == 0:
			self.view.image_files_list.add_button["state"] = "normal"
			self.view.image_files_list.del_button["state"] = "disabled"
			self.view.roi_files_list.add_button["state"] = "normal"
			self.view.roi_files_list.del_button["state"] = "disabled"

		if len(roi_files_cursor) > 0:
			self.view.image_files_list.add_button["state"] = "disabled"
			self.view.roi_files_list.add_button["state"] = "disabled"
			self.view.image_files_list.del_button["state"] = "disabled"
			self.view.roi_files_list.del_button["state"] = "normal"

		elif len(image_files_cursor) > 0:
			self.view.image_files_list.add_button["state"] = "disabled"
			self.view.roi_files_list.add_button["state"] = "disabled"
			self.view.image_files_list.del_button["state"] = "normal"
			self.view.roi_files_list.del_button["state"] = "disabled"

	def image_files_list_event(self, event):
		image_files_cursor = self.view.image_files_list.listbox.curselection()
		roi_files_cursor = self.view.roi_files_list.listbox.curselection()
		all_files_cursor = self.view.all_files_list.listbox.curselection()
		if len(all_files_cursor) > 0:
			self.view.image_files_list.add_button["state"] = "normal"
			self.view.roi_files_list.add_button["state"] = "normal"
			self.view.image_files_list.del_button["state"] = "disabled"
			self.view.roi_files_list.del_button["state"] = "disabled"

		elif len(roi_files_cursor) > 0:
			self.view.image_files_list.add_button["state"] = "disabled"
			self.view.roi_files_list.add_button["state"] = "disabled"
			self.view.image_files_list.del_button["state"] = "disabled"
			self.view.roi_files_list.del_button["state"] = "normal"

		elif len(image_files_cursor) > 0:
			self.view.image_files_list.add_button["state"] = "disabled"
			self.view.roi_files_list.add_button["state"] = "disabled"
			self.view.image_files_list.del_button["state"] = "normal"
			self.view.roi_files_list.del_button["state"] = "disabled"

	def image_del_button(self):
		self.file_list_modified = True

		image_files_cursor = self.view.image_files_list.listbox.curselection()
		if len(image_files_cursor) == 0:
			return True
		index_to_del = image_files_cursor[0]
		self.view.image_files_list.listbox.delete(index_to_del)
		self.model.image_files_list.pop(index_to_del)

		if len(self.model.image_files_list) == 0:
			self.view.next_button["state"] = "disabled"
			return True

		if self.model.check_file_list_valid():
			self.view.next_button['state'] = 'normal'
		else:
			self.view.next_button['state'] = 'disabled'

		if index_to_del >= len(self.model.image_files_list):
			index = len(self.model.image_files_list)-1
		else:
			index = index_to_del

		self.view.image_files_list.listbox.selection_set(index)
		self.view.image_files_list.listbox.see(index)
		self.view.image_files_list.listbox.activate(index)
		self.view.image_files_list.listbox.selection_anchor(index)

	def roi_files_list_event(self, event):
		all_files_cursor = self.view.all_files_list.listbox.curselection()
		image_files_cursor = self.view.image_files_list.listbox.curselection()
		roi_files_cursor = self.view.roi_files_list.listbox.curselection()

		if len(all_files_cursor) > 0:
			self.view.image_files_list.add_button["state"] = "normal"
			self.view.roi_files_list.add_button["state"] = "normal"
			self.view.image_files_list.del_button["state"] = "disabled"
			self.view.roi_files_list.del_button["state"] = "disabled"

		elif len(image_files_cursor) > 0:
			self.view.image_files_list.add_button["state"] = "disabled"
			self.view.roi_files_list.add_button["state"] = "disabled"
			self.view.image_files_list.del_button["state"] = "normal"
			self.view.roi_files_list.del_button["state"] = "disabled"
		
		elif len(roi_files_cursor) > 0:
			self.view.image_files_list.add_button["state"] = "disabled"
			self.view.roi_files_list.add_button["state"] = "disabled"
			self.view.image_files_list.del_button["state"] = "disabled"
			self.view.roi_files_list.del_button["state"] = "normal"

	def roi_del_button(self):
		self.file_list_modified = True
		roi_files_cursor = self.view.roi_files_list.listbox.curselection()
		if len(roi_files_cursor) == 0:
			return True

		index_to_del = roi_files_cursor[0]
		self.view.roi_files_list.listbox.delete(index_to_del)
		self.model.roi_files_list.pop(index_to_del)

		if len(self.model.roi_files_list) == 0:
			self.view.next_button["state"] = "disabled"
			return True

		if self.model.check_file_list_valid():
			self.view.next_button['state'] = 'normal'
		else:
			self.view.next_button['state'] = 'disabled'

		if index_to_del >= len(self.model.roi_files_list):
			index = len(self.model.roi_files_list)-1
		else:
			index = index_to_del

		self.view.roi_files_list.listbox.selection_set(index)
		self.view.roi_files_list.listbox.see(index)
		self.view.roi_files_list.listbox.activate(index)
		self.view.roi_files_list.listbox.selection_anchor(index)

	def option_event(self):
		self.view.next_button['state'] = 'disabled'
		self.view.save_load_label['text'] = ''
		for options_object in [self.view.setting,self.view.voxelSetting,self.view.featureClass]:
			for i, option in enumerate(options_object.options_list):
				show_flag = options_object.options_vars[option].get()
				option_dict = options_object.options_dicts[i]

				if option_dict['has_entry'] or option_dict['has_entries']:
					if show_flag == 1:
						options_object.show_entries(option)
					else:
						options_object.hide_entries(option)		

				if option == 'voxelBased':
					if show_flag:
						self.view.featureClass.options_vars['shape'].set(0)
						self.view.featureClass.options_buttons['shape']['state'] = 'disabled'
						self.view.voxelSetting.activate_all()
					else:
						self.view.featureClass.options_buttons['shape']['state'] = 'normal'
						for voxelSetting_option in self.view.voxelSetting.options_list:
							if voxelSetting_option == 'voxelBased':
								self.view.voxelSetting.options_vars[voxelSetting_option].set(0)
							else:
								self.view.voxelSetting.options_buttons[voxelSetting_option]['state'] = 'disabled'
								self.view.voxelSetting.options_vars[voxelSetting_option].set(0)
								self.view.voxelSetting.hide_entries(voxelSetting_option)

				elif option == 'shape':
					if show_flag:
						self.view.voxelSetting.deactivate_all()
						self.view.voxelSetting.options_vars['voxelBased'].set(0)
					else:
						self.view.voxelSetting.options_buttons['voxelBased']['state'] = 'normal'

	def directory_radio(self):
		self.directory_radio_flag = True
		selection = "You selected the option " + str(self.view.directory_radio.radio_var.get())

		if self.directory_browse_flag == True:
			self.view.next_button['state'] = 'normal'

		if self.view.directory_radio.radio_var.get()==1:
			if self.model.directory_setup == 'diff':
				self.file_list_modified = False
			self.model.directory_setup = 'same'

		elif self.view.directory_radio.radio_var.get()==2:
			if self.model.directory_setup == 'same':
				self.file_list_modified = False
				self.model.image_path = ''
				self.model.roi_path = ''
			self.model.directory_setup = 'diff'



import tkinter.filedialog
import tkinter as tk
import pandas as pd
import SimpleITK as sitk
import numpy as np

import radiomics
import threading
import logging
import shutil
import yaml
import csv
import six
import os

from radiomics import getFeatureClasses
from radiomics.featureextractor import RadiomicsFeatureExtractor
from multiprocessing import cpu_count, Pool
from collections import OrderedDict
from datetime import datetime

class Model():
	def __init__(self):
		self.data_path = ''
		self.image_path = ''
		self.roi_path = ''
		self.default_data_path = os.getcwd()
		self.default_image_path = ''
		self.default_roi_path = ''
		self.initial_dir = self.default_data_path
		self.directory_setup = ''
		self.subject_list = []
		self.all_files_list = []
		self.image_files_list = []
		self.roi_files_list = []
		self.file_list_valid = False
		self.data_files_valid = False
		self.yaml_dicts = {'imageType':{'Original': {}}}
		self.yaml_file = ''
		self.subject_files = {}
		self.logs_file = ''
		self.output_file = ''

	def read_meta_params(self):
		meta_params_found = False
		lines = []
		with open(self.yaml_file, 'r') as yaml_file:
			for line in yaml_file:  
				# checking string is present in line or not
				if meta_params_found == True:
					lines.append(line)
				elif '# meta-parameters (DO NOT EDIT)' in line:
					meta_params_found = True

			if meta_params_found:
				voxelBased_line = lines[0]
				downSampling_line = lines[1]

				voxelBased = voxelBased_line[voxelBased_line.index(':')+1:].strip() == 'true'
				downSampling = downSampling_line[downSampling_line.index(':')+1:].strip() == 'true'

				if voxelBased == True:
					if self.yaml_dicts['voxelSetting'] is None:
						self.yaml_dicts['voxelSetting'] = {}
					self.yaml_dicts['voxelSetting']['voxelBased'] = True
					self.yaml_dicts['voxelSetting']['downSampling'] = True
				
					if downSampling == False:
						self.yaml_dicts['voxelSetting']['downSampling'] = False

	def browse_dir(self, step_name):
		dir_path = ''
		if step_name == 'Step 2':
			data_path = tk.filedialog.askdirectory(initialdir = self.default_data_path)
			if len(data_path) > 0:
				self.data_path = data_path
				self.default_data_path = data_path
				self.initial_dir = os.path.split(data_path)[0]
				self.init_subject_list()
			dir_path = self.data_path

		elif step_name == 'Step 3':
			image_path = tk.filedialog.askdirectory(initialdir = self.default_image_path)
			if len(image_path) > 0:
				self.image_path = image_path
				self.init_image_files_list()
			dir_path = self.image_path

		elif step_name == 'Step 4':
			roi_path = tk.filedialog.askdirectory(initialdir = self.default_roi_path)
			if len(roi_path) > 0:
				self.roi_path = roi_path
				self.init_roi_files_list()
				self.init_all_files_list()
			dir_path = self.roi_path

		return dir_path

	def load(self):
		open_file = tk.filedialog.askopenfile(mode="r",initialdir = self.initial_dir,filetypes=[("YAML file", "*.yaml")])
		if open_file is not None:
			self.yaml_file = open_file.name
			self.yaml_dicts = {}
			with open(self.yaml_file, 'r') as stream:
				try:
					self.yaml_dicts = yaml.safe_load(stream)
					self.read_meta_params()
				except yaml.YAMLError as exc:
					print(exc)
		else:
			self.yaml_file = ''
			self.yaml_dicts = {}

	def save(self,setting_dict,voxelSetting_dict,featureClass_dict):
		self.setting_dict = setting_dict
		self.voxelSetting_dict = voxelSetting_dict
		self.featureClass_dict = featureClass_dict

		self.yaml_dicts['setting'] = setting_dict
		self.yaml_dicts['voxelSetting'] = voxelSetting_dict
		self.yaml_dicts['featureClass'] = featureClass_dict

		save_file = tk.filedialog.asksaveasfile(defaultextension='.yaml',initialdir = self.initial_dir,filetypes=[("YAML file", "*.yaml")])
		meta_params = ['\n\n# meta-parameters (DO NOT EDIT)\n']
		if save_file is not None:
			self.yaml_file = save_file.name
			with open(self.yaml_file, 'a') as the_file:
				
				the_file.write('imageType:\n  Original: {}\n\n')
				if len(self.setting_dict) > 0:
					# self.yaml_dicts['setting'] = self.setting_dict
					the_file.write('setting:\n')
					for key,val in self.setting_dict.items():
						if isinstance(val, bool):
							if val == True:
								the_file.write('  '+key+': '+'true'+'\n')
							else:
								the_file.write('  '+key+': '+'false'+'\n')
						else:
							the_file.write('  '+key+': '+str(val)+'\n')
					the_file.write('\n')

				
				if 'voxelBased' in list(self.voxelSetting_dict.keys()):
					self.yaml_dicts['voxelSetting'] = self.voxelSetting_dict
					if 'downSampling' not in list(self.voxelSetting_dict.keys()):
						self.voxelSetting_dict['downSampling'] = False

					for key,val in self.voxelSetting_dict.items():

						if key == 'voxelBased':
							if val == True:
								the_file.write('voxelSetting:\n')
								meta_params.append('# voxelBased: true\n')	
							else:
								meta_params.append('# voxelBased: false\n')
						elif key == 'downSampling':
							if val == False:
								meta_params.append('# downSampling: false\n')	
							else:
								meta_params.append('# downSampling: true\n')
						else:
							if isinstance(val, bool):
								
								if val == True:
									the_file.write('  '+key+': '+'true'+'\n')
								else:
									the_file.write('  '+key+': '+'false'+'\n')
							else:
								the_file.write('  '+key+': '+str(val)+'\n')
					the_file.write('\n')
				else:
					the_file.write('\n')

				if len(self.featureClass_dict) > 0:
					the_file.write('featureClass:\n')
					for key,val in self.featureClass_dict.items():
						the_file.write('  '+key+':\n')
					the_file.write('\n')
				
				the_file.write(''.join(meta_params))

	def init_subject_list(self):
		if len(self.data_path) > 0:
			self.subject_list = [dir_item for dir_item in os.listdir(self.data_path) if os.path.isdir(os.path.join(self.data_path,dir_item))]

	def init_roi_files_list(self):
		if len(self.roi_path) > 0:
			self.roi_files_list = [dir_item for dir_item in os.listdir(self.roi_path) if dir_item[0] != '.']# if (dir_item[-3:]=='nii' or dir_item[-6:]=='nii.gz')]

	def init_image_files_list(self):
		if len(self.image_path) > 0:
			self.image_files_list = [dir_item for dir_item in os.listdir(self.image_path) if dir_item[0] != '.']# if (dir_item[-3:]=='nii' or dir_item[-6:]=='nii.gz')]

	def init_all_files_list(self):
		self.all_files_list = list(set(self.image_files_list+self.roi_files_list))
		pass

	def check_file_list_valid(self):
		if len(set(self.image_files_list).intersection(self.roi_files_list)) > 0:
			self.file_list_valid = False
		elif len(self.image_files_list) == 0 or len(self.roi_files_list) == 0:
			self.file_list_valid = False
		else:
			self.file_list_valid = True

		return self.file_list_valid

	def validate_directory(self):
		if self.directory_setup == 'same':

			subject_files = {}
			for subject in self.subject_list:
				subject_files[subject] = {}
				for image_file in self.image_files_list:
					image_path = os.path.join(self.data_path,subject,image_file)

					if os.path.isfile(image_path):
						subject_files[subject][image_file] = image_path
					else:
						return False, file_path

				for roi_file in self.roi_files_list:
					roi_path = os.path.join(self.data_path,subject,roi_file)

					if os.path.isfile(roi_path):
						subject_files[subject][roi_file] = roi_path
					else:
						return False, file_path
			
			self.subject_files = subject_files
			return True, ''

		elif self.directory_setup == 'diff':

			subject_files = {}
			for subject in self.subject_list:
				subject_files[subject] = {}

				image_folder = os.path.join(self.data_path,subject,os.path.split(self.image_path)[1])
				for image_file in self.image_files_list:
					image_path = os.path.join(image_folder,image_file)

					if os.path.isfile(image_path):
						subject_files[subject][image_file] = image_path
					else:
						return False, image_path

				roi_folder = os.path.join(self.data_path,subject,os.path.split(self.roi_path)[1])
				for roi_file in self.roi_files_list:
					roi_path = os.path.join(roi_folder,roi_file)

					if os.path.isfile(roi_path):
						subject_files[subject][roi_file] = roi_path
					else:
						return False, roi_path

			self.subject_files = subject_files
			return True, ''

	def init_output_file(self):
		initial_dir = os.path.split(self.data_path)[0]
		initial_name = os.path.split(self.data_path)[1]+'_features.csv'
		output_file = tk.filedialog.asksaveasfile(defaultextension='.csv',initialdir=initial_dir,filetypes=[("CSV file", "*.csv")])
		
		if output_file is not None:
			self.output_file = output_file.name
			self.root_dir = os.path.split(self.output_file)[0]
			self.temp_dir = os.path.join(self.root_dir,'_TEMP')

	def run(self,subject):
		# Instantiate Radiomics Feature extractor
		extractor = RadiomicsFeatureExtractor(self.yaml_file)

		#self.parent.update_idletasks()
		ptLogger = logging.getLogger('radiomics.batch')
		feature_vector = OrderedDict({'subject_ID':subject})

		# set thread name to subject ID
		threading.current_thread().name = subject 

		subject_paths = self.subject_files[subject]

		for image in self.image_files_list:
			image_path = subject_paths[image]
			image_name = image[0:image.index('.')]
			
			for roi in self.roi_files_list:
				roi_path = subject_paths[roi]
				roi_name = roi[0:roi.index('.')]

				try:
					filename = r'features_'+subject+ '_' + image_name + '_' + roi_name + '.csv'
					output_filename = os.path.join(self.root_dir, self.temp_dir, filename)

					if os.path.isfile(output_filename):
						# Output already generated, load result (prevents re-extraction in case of interrupted process)
						with open(output_filename, 'w') as outputFile:
							reader = csv.reader(outputFile)
							headers = reader.rows[0]
							values = reader.rows[1]
							feature_vector = OrderedDict(zip(headers, values))

						ptLogger.info('Image: %s,  ROI: %s  already processed...', image_name, roi_name)

					else:
						t = datetime.now()

						pyrads_result = extractor.execute(image_path, roi_path)
						pyrads_list = []

						for key,val in pyrads_result.items():
							pyrads_key = image_name + '_' + roi_name + '_' + key
							pyrads_list.append((pyrads_key,val))

						pyrads_dict = OrderedDict(pyrads_list[22:])
						feature_vector.update(pyrads_dict)

						# Store results in temporary separate files to prevent write conflicts
						# This allows for the extraction to be interrupted. Upon restarting, already processed cases are found in the
						# TEMP_DIR directory and loaded instead of re-extracted
						with open(output_filename, 'w') as outputFile:
							writer = csv.DictWriter(outputFile, fieldnames=list(feature_vector.keys()), lineterminator='\n')
							writer.writeheader()
							writer.writerow(feature_vector)
							# Display message
						delta_t = datetime.now() - t

						ptLogger.info('Subject: %s, Image: %s,  ROI: %s  processed in %s', subject,image_name,roi_name,delta_t)

				except Exception:
					ptLogger.error('Feature extraction failed!', exc_info=True)

		return feature_vector

	def extract_segment_features(self,logger):
		# Ensure the entire extraction is handled on 1 thread
		#####################################################
		sitk.ProcessObject_SetGlobalDefaultNumberOfThreads(1)
		# Set up the pool processing
		############################
		logger.info('pyradiomics version: %s', radiomics.__version__)
		logger.info('Loaded %d jobs', len(self.subject_list))

		# Make output directory
		if os.path.isdir(os.path.join(self.root_dir, self.temp_dir)):
			shutil.rmtree(os.path.join(self.root_dir, self.temp_dir))			
		logger.info('Creating temporary output directory %s', os.path.join(self.root_dir, self.temp_dir))
		os.mkdir(os.path.join(self.root_dir, self.temp_dir))

		NUM_OF_WORKERS = cpu_count() - 1  # Number of processors to use, keep one processor free for other work
		if NUM_OF_WORKERS < 1:  # in case only one processor is available, ensure that it is used
			NUM_OF_WORKERS = 1

		logger.info('Starting parallel pool with %d workers out of %d CPUs', NUM_OF_WORKERS, cpu_count())

		# Running the Pool
		pool = Pool(NUM_OF_WORKERS)
		results = pool.map(self.run, self.subject_list)

		try:
			# Store all results into 1 file
			with open(self.output_file, mode='w') as outputFile:
				writer = csv.DictWriter(outputFile,
					fieldnames=list(results[0].keys()),
					restval='',
					extrasaction='raise',  # raise error when a case contains more headers than first case
					lineterminator='\n')
				writer.writeheader()
				writer.writerows(results)

			# if REMOVE_TEMP_DIR:
			logger.info('Removing temporary directory %s (contains individual case results files)',
									os.path.join(self.root_dir, self.temp_dir))
			shutil.rmtree(os.path.join(self.root_dir, self.temp_dir))


		except Exception:
			logger.error('Error storing results into single file!', exc_info=True)

	def extract_voxel_features(self,logger):
		NUM_OF_WORKERS = cpu_count() - 1  # Number of processors to use, keep one processor free for other work
		if NUM_OF_WORKERS < 1:  # in case only one processor is available, ensure that it is used
			NUM_OF_WORKERS = 1

		# Make output directory
		if os.path.isdir(os.path.join(self.root_dir, self.temp_dir)):
			shutil.rmtree(os.path.join(self.root_dir, self.temp_dir))			
		logger.info('Creating temporary output directory %s', os.path.join(self.root_dir, self.temp_dir))
		os.mkdir(os.path.join(self.root_dir, self.temp_dir))

		# Ensure the entire extraction is handled on 1 thread
		#####################################################
		sitk.ProcessObject_SetGlobalDefaultNumberOfThreads(1)
		# Set up the pool processing
		############################
		logger.info('pyradiomics version: %s', radiomics.__version__)

		# Initialize feature extractor using the settings file
		extractor = RadiomicsFeatureExtractor(self.yaml_file)
		featureClasses = getFeatureClasses()

		features_dict = {}

		for feature_class, features in six.iteritems(extractor.enabledFeatures):
			features_dict[feature_class] = []

			if features is None or len(features) == 0:
				features = [f for f, deprecated in six.iteritems(featureClasses[feature_class].getFeatureNames()) if not deprecated]

			for f in features:
				features_dict[feature_class].append(f)

		output_dict = {}
		first_subject = True
		for subject in self.subject_list:
			subject_paths = self.subject_files[subject]
			output_dict[subject] = {}

			output_filelist = {}
			for roi in self.roi_files_list:
				roi_path = subject_paths[roi]
				roi_name = roi[0:roi.index('.')]
				output_filelist[roi_name] = {}

				_, file_extension = os.path.splitext(roi_path)
				if file_extension == '.gz':
					file_extension = '.nii.gz'
				filename = subject + '_' + roi_name + '_firstorder_Mean' + file_extension # this is important
				roi_samples_path = os.path.join(self.root_dir, self.temp_dir, filename)

				extractor = RadiomicsFeatureExtractor(self.yaml_file)
				extractor.disableAllFeatures()
				extractor.enabledFeatures = {'firstorder':['Mean']}
				extractor.enableFeaturesByName()

				featureVector = extractor.execute(roi_path,roi_path, voxelBased=True)

				for featureName, featureValue in six.iteritems(featureVector):
					if isinstance(featureValue, sitk.Image):
						roi_samples = featureValue				
						if 'downSampling' in list(self.yaml_dicts['voxelSetting'].keys()):
							if self.yaml_dicts['voxelSetting']['downSampling'] == True:
								if 'kernelRadius' in list(self.yaml_dicts['voxelSetting'].keys()):
									r = self.yaml_dicts['voxelSetting']['kernelRadius']
								else:
									r = 2
								stride = 2*r + 1

								roi = sitk.GetArrayFromImage(roi_samples)

								if 'force2D' in list(self.yaml_dicts['setting'].keys()):
									force2D = self.yaml_dicts['setting']['force2D']
								else:
									force2D = False

								offsets_checked = []
								num_samples = []

								for offset_i in range(0,stride):
									for offset_j in range(0,stride):
										if force2D == False:
											for offset_k in range(0,stride):
												offset = (offset_i,offset_j,offset_k)
												downSampling = 0*roi
												downSampling[0:-1:stride,0:-1:stride,0:-1:stride] = 1
												np.roll(downSampling,offset,(0,1,2))
												samples = np.multiply(roi,downSampling)
												
												offsets_checked.append(offset)
												num_samples.append(np.sum(np.ravel(samples)))
										else:
											offset = (offset_i,offset_j,0)
											downSampling = 0*roi
											downSampling[0:-1:stride,0:-1:stride,:] = 1
											np.roll(downSampling,offset,(0,1,2))
											samples = np.multiply(roi,downSampling)
											
											offsets_checked.append(offset)
											num_samples.append(np.sum(np.ravel(samples)))

								ind = num_samples.index(max(num_samples))
								offset = offsets_checked[ind]
								
								downSampling = 0*roi
								if force2D == False:
									downSampling[0:-1:stride,0:-1:stride,0:-1:stride] = 1
									np.roll(downSampling,offset,(0,1,2))
								else:
									downSampling[0:-1:stride,0:-1:stride,:] = 1
									np.roll(downSampling,offset,(0,1,2))

								roi_samples_nda = np.multiply(roi,downSampling)
								roi_samples = sitk.GetImageFromArray(roi_samples_nda)

						sitk.WriteImage(roi_samples, roi_samples_path)

				for image in self.image_files_list:
					image_path = subject_paths[image]
					image_name = image[0:image.index('.')]

					output_filelist[roi_name][image_name] = {}

					for feature_class, feature_list in features_dict.items():

						output_filelist[roi_name][image_name][feature_class] = {}
						cases = []	
						for feature in feature_list:
							case = {'subject_ID':subject,'image_name':image_name,'image_path':image_path,'roi_name':roi_name,'roi_path':roi_path}
							case['roi_samples'] = roi_samples_path
							case['feature_class'] = feature_class
							case['feature'] = feature
							cases.append(case)

							filename = case['subject_ID']+'_'+case['image_name']+'_'+case['roi_name']+'_'+case['feature_class']+'_'+case['feature']+'.csv'
							output_filelist[roi_name][image_name][feature_class][feature] = filename# os.path.join(self.root_dir, self.temp_dir, filename))

						logger.info('Starting parallel pool with %d workers out of %d CPUs to compute %d %s feature maps', NUM_OF_WORKERS, cpu_count(), len(feature_list), feature_class)
						pool = Pool(NUM_OF_WORKERS)
						results = pool.map(self.run_voxel, cases)

			# subject_ID_added = False
			for image in self.image_files_list:
				image_name = image[0:image.index('.')]

				for roi in self.roi_files_list:
					roi_name = roi[0:roi.index('.')]

					for feature_class, feature_list in features_dict.items():

						for feature in feature_list:
							csv_file = output_filelist[roi_name][image_name][feature_class][feature]
							csv_path = os.path.join(self.root_dir,self.temp_dir,csv_file)
							df = pd.read_csv(csv_path)
							feature_name = df.columns[0]
							output_dict[subject][feature_name] = df.values[:,0].tolist()

			if first_subject == True:
				subject_df = pd.DataFrame.from_dict(output_dict[subject])
				subject_df = pd.concat([pd.Series([subject]*subject_df.shape[0],name='subject_ID'),subject_df],axis=1)
				first_subject = False
			else:
				subject_df_i = pd.DataFrame.from_dict(output_dict[subject])
				subject_df_i = pd.concat([pd.Series([subject]*subject_df_i.shape[0],name='subject_ID'),subject_df_i],axis=1)
				subject_df = pd.concat([subject_df,subject_df_i],axis=0)

		subject_df.to_csv(self.output_file,index=False)

		# if REMOVE_TEMP_DIR:
		logger.info('Removing temporary directory %s (contains individual case results files)',
								os.path.join(self.root_dir, self.temp_dir))
		shutil.rmtree(os.path.join(self.root_dir, self.temp_dir))

	def run_voxel(self,case):
		# Instantiate Radiomics Feature extractor
		extractor = RadiomicsFeatureExtractor(self.yaml_file)
		extractor.disableAllFeatures()
		extractor.enabledFeatures = {case['feature_class']:[case['feature']]}
		extractor.enableFeaturesByName()

		ptLogger = logging.getLogger('radiomics.batch')

		# set thread name to feature class
		threading.current_thread().name = case['feature_class']

		try:
			filename = case['subject_ID']+'_'+case['image_name']+'_'+case['roi_name']+'_'+case['feature_class']+'_'+case['feature']+'.csv'
			output_filename = os.path.join(self.root_dir, self.temp_dir, filename)

			if os.path.isfile(output_filename):
			# Output already generated, load result (prevents re-extraction in case of interrupted process)
				ptLogger.info('Subject: %s, Image: %s,  ROI: %s,  Feature: %s  already processed...',case['subject_ID'],case['image_name'],case['roi_name'],case['feature'])
			else:
				t = datetime.now()
				featureVector = extractor.execute(case['image_path'], case['roi_path'], voxelBased=True)

				image = None
				for featureName, featureValue in six.iteritems(featureVector):
					if isinstance(featureValue, sitk.Image):
						image = featureValue

						roi_samples = sitk.GetArrayFromImage(sitk.ReadImage(case['roi_samples']))
						image_vals = sitk.GetArrayFromImage(image)
						image_sampled = np.ravel(image_vals[roi_samples == 1]) # TODO: add label support

						feature_name = case['image_name']+'_'+case['roi_name']+'_'+case['feature_class']+'_'+case['feature']
						feature_values = pd.Series(data=image_sampled, name=feature_name)

						with open(output_filename, 'w') as outputFile:
							feature_values.to_csv(output_filename,index=False)
							delta_t = datetime.now() - t
							ptLogger.info('Subject: %s, Image: %s,  ROI: %s,  Feature: %s  processed in %s',case['subject_ID'],case['image_name'], case['roi_name'],case['feature'],delta_t)
		
		except Exception:
			ptLogger.error('Feature extraction failed!', exc_info=True)

	def extract_features(self):
		threading.current_thread().name = 'Main'
		self.logs_file = os.path.join(self.root_dir,'pyrad_log.txt')

		rLogger = radiomics.logger
		logHandler = logging.FileHandler(filename=self.logs_file, mode='w')
		logHandler.setLevel(logging.INFO)
		logHandler.setFormatter(logging.Formatter('%(levelname)-.1s: (%(threadName)s) %(name)s: %(message)s'))
		rLogger.addHandler(logHandler)

		outputhandler = rLogger.handlers[0]  # Handler printing to the output
		outputhandler.setFormatter(logging.Formatter('[%(asctime)-.19s] (%(threadName)s) %(name)s: %(message)s'))
		outputhandler.setLevel(logging.INFO)  # Ensures that INFO messages are being passed to the filter
		outputhandler.addFilter(info_filter('radiomics.batch'))

		logging.getLogger('radiomics.batch').debug('Logging init')
		logger = logging.getLogger('radiomics.batch')

		if 'voxelSetting' in list(self.yaml_dicts.keys()):
			self.extract_voxel_features(logger)
		else:
			self.extract_segment_features(logger)

class info_filter(logging.Filter):
	def __init__(self, name):
		super(info_filter, self).__init__(name)
		self.level = logging.WARNING

	def filter(self, record):
		if record.levelno >= self.level:
			return True
		if record.name == self.name and record.levelno >= logging.INFO:
			return True
		return False




import tkinter as tk
import tkinter.font as tkFont
import tkinter.scrolledtext as ScrolledText
from tkinter import StringVar, IntVar

class View():
	def __init__(self, parent, controller):
		self.step = 'Step 1'
		self.controller = controller


		self.init_params(self.controller)

		self.wiz_steps = WizList(parent)
		self.instruction = self.init_instruction(parent)
		self.path_entry = self.init_path_entry(parent)	
		self.save_load_label = self.init_save_load_label(parent)			
		self.save_load_label_text = ''
		
		self.next_button = self.init_button(parent,self.params['next_button']) 
		self.back_button = self.init_button(parent,self.params['back_button'])
		self.load_button = self.init_button(parent,self.params['load_button'])
		self.save_button = self.init_button(parent,self.params['save_button'])
		self.browse_dir_button = self.init_button(parent,self.params['browse_dir_button'])

		self.all_files_list = FileList(parent,self.params['all_files_list'],self.controller)
		self.image_files_list = FileList(parent,self.params['image_files_list'],self.controller)
		self.roi_files_list = FileList(parent,self.params['roi_files_list'],self.controller)
		self.directory_radio = RadioSelect(parent,self.params['directory_radio'],self.controller)

		self.logs = self.init_logs(parent,self.params['logs'])

		self.hide_buttons()
		self.hide_path_entry()
		self.roi_files_list.hide()
		self.image_files_list.hide()
		self.all_files_list.hide()

		self.setting = ExtractionOptions(parent,self.params['setting'],self.controller)
		self.voxelSetting = ExtractionOptions(parent,self.params['voxelSetting'],self.controller)
		self.featureClass = ExtractionOptions(parent,self.params['featureClass'],self.controller)
		self.init_voxelSetting()
		
		self.wiz_steps.activate(self.step)
		self.show_step1(parent, controller)
		self.next_button['state'] = 'disabled'

	def init_params(self,controller):
		# logging parameters
		logs = {'font':'Arial','fontsize':12,'x':250,'y':160,'width':500,'height':300,'text':'logs'}

		# button parameters
		next_button = {'font':'Arial','fontsize':12,'x':730,'y':520,'width':64,'height':24,'text':'Next','command':controller.next}
		back_button = {'font':'Arial','fontsize':12,'x':640,'y':520,'width':64,'height':24,'text':'Back','command':controller.back}
		load_button = {'font':'Arial','fontsize':12,'x':250,'y':350,'width':64,'height':24,'text':'Load','command':controller.load}
		save_button = {'font':'Arial','fontsize':12,'x':250+64+24,'y':350,'width':64,'height':24,'text':'Save','command':controller.save}
		browse_dir_button = {'font':'Arial','fontsize':12,'x':730,'y':123,'width':64,'height':24,'text':'Browse','command':controller.browse_dir}
		
		# directory setup parameters
		directory_radio = {'font':'Arial','fontsize':12,'x':250,'y':160,'width':232,'height':100, 'dx2':232+12,  \
						'label_fontsize':16,'label_y':160,'label_height':26, \
						'button_font':'Arial','button_fontsize':12,'button_y':None,'button_width':None, \
						'text1':'Images & ROIs in Same Folder','text2':'Images & ROIs in Subfolders'}
		
		# path entry parameters
		path_entry = {'font':'Arial','fontsize':12,'x':250,'y':120,'width':472,'height':30}

		# save / load label
		save_load_label = {'font':'Arial','fontsize':12,'x':250,'y':350+32,'width':None,'height':None,'text':''}
		
		# file list object parameters
		all_files_list = {'font':'Arial','fontsize':12,'x':250,'y':160,'width':175,'height':100,  \
						'label_fontsize':16,'label_y':138,'label_height':26, \
						'button_font':'Arial','button_fontsize':12,'button_y':None,'button_width':None, \
						'button_height':24, 'add_x':None, 'del_x':None,'text':'All Files'}
		
		image_files_list = {'font':'Arial','fontsize':12,'x':437,'y':160,'width':175,'height':100,  \
						'label_fontsize':16,'label_y':138,'label_height':26, \
						'button_font':'Arial','button_fontsize':12,'button_y':None,'button_width':64, \
						'button_height':24, 'add_x':437, 'del_x':437+68,'text':'Image Files'}
		
		roi_files_list = {'font':'Arial','fontsize':12,'x':624,'y':160,'width':175,'height':100,  \
						'label_fontsize':16,'label_y':138,'label_height':26, \
						'button_font':'Arial','button_fontsize':12,'button_y':None,'button_width':64, \
						'button_height':24, 'add_x':624, 'del_x':624+68,'text':'ROI Files'}

		# setting parameters
		force2D = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,"dy":12,'default':None}
		normalize = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,'dy':12,'default':None}
		binWidth = {'has_entry':True,'entry_type':float(25.0),'dx':90, 'width':32,'height':12+4, 'has_entries':False,'dy':12,'default':25.0}
		removeOutliers = {'has_entry':True,'entry_type':float(3.0),'dx':135, 'width':32,'height':12+4,'has_entries':False,'dy':12,'default':3.0}
		resampledPixelSpacing = {'has_entry':False,'entry_type':float(3.0),'dx':[24,24+32+4,24+2*(32+4)], 'width':32,'height':12+4,'has_entries':True,'dy':12+16,'default':[3.0,3.0,3.0]}

		setting_list = [('force2D',force2D),('normalize',normalize),('binWidth',binWidth),\
						('removeOutliers',removeOutliers),('resampledPixelSpacing',resampledPixelSpacing)]

		setting = {'font':'Arial','lb_fontsize':16,'x':250,'y':160,'lb_width':150,'lb_height':32+4, \
						'btn_fontsize':12,'btn_x':250,'btn_y':160+24,'btn_dy':12+16, 'btn_width':150,'btn_height':12+16, \
						'option_list':setting_list,'text':'Settings'}

		# voxelSetting parameters
		voxelBased = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,"dy":12,'default':None}
		maskedKernel = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,'dy':12,'default':None}
		downSampling = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,'dy':12,'default':None}
		kernelRadius = {'has_entry':True,'entry_type':int(2),'dx':90+32, 'width':32,'height':12+4, 'has_entries':False,'dy':12,'default':2}
		voxelBatch = {'has_entry':True,'entry_type':int(10000),'dx':90+16, 'width':64,'height':12+4,'has_entries':False,'dy':12,'default':10000}
		initValue = {'has_entry':True,'entry_type':float(0.0),'dx':90, 'width':32,'height':12+4,'has_entries':False,'dy':12,'default':0.0}

		voxelSetting_list = [('voxelBased',voxelBased),('maskedKernel',maskedKernel),('downSampling',downSampling),\
							('kernelRadius',kernelRadius),('voxelBatch',voxelBatch),('initValue',initValue)]

		voxelSetting = {'font':'Arial','lb_fontsize':16,'x':250+210,'y':160,'lb_width':150,'lb_height':32+4, \
						'btn_fontsize':12,'btn_x':250+210,'btn_y':160+24,'btn_dy':12+16, 'btn_width':150,'btn_height':12+16, \
						'option_list':voxelSetting_list,'text':'Voxel Settings'}

		# featureClass parameters
		shape = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,"dy":12,'default':None}
		firstorder = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,"dy":12,'default':None}
		glcm = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,"dy":12,'default':None}
		glszm = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,"dy":12,'default':None}
		glrlm = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,"dy":12,'default':None}
		ngtdm = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,"dy":12,'default':None}
		gldm = {'has_entry':False,'dx':None, 'width':None,'height':12+4,'has_entries':False,"dy":12,'default':None}

		featureClass_list =[('shape',shape),('firstorder',firstorder),('glcm',glcm),('glszm',glszm),('glrlm',glrlm),('ngtdm',ngtdm),('gldm',gldm)]

		featureClass = {'font':'Arial','lb_fontsize':16,'x':250+390,'y':160,'lb_width':150,'lb_height':32+4, \
						'btn_fontsize':12,'btn_x':250+390,'btn_y':160+24,'btn_dy':12+16, 'btn_width':150,'btn_height':12+16, \
						'option_list':featureClass_list,'text':'Feature Classes'}

		# pack parameters
		self.params= {'path_entry':path_entry,'browse_dir_button':browse_dir_button, 'back_button':back_button, \
						'next_button':next_button, 'load_button':load_button,'save_button':save_button, \
						'all_files_list':all_files_list, 'image_files_list':image_files_list, 'roi_files_list':roi_files_list, \
						'setting':setting,'voxelSetting':voxelSetting,'featureClass':featureClass, \
						'save_load_label':save_load_label,'directory_radio':directory_radio,'logs':logs}

	def init_logs(self,parent,params):
		logs = ScrolledText.ScrolledText(parent, state='disabled')
		# logs['justify'] = 'left'
		# label['text'] = 'Browse'
		# label['bg'] = '#F0F0F0'
		# label['fg'] = '#000000'
		#logs['anchor'] = 'nw'
		logs['font'] = tkFont.Font(family=params['font'],size=params['fontsize'])
		logs.place(x=params['x'],y=params['y'],width=params['width'],height=params['height'])

		return logs

	def init_voxelSetting(self):
		for voxelSetting_option in self.voxelSetting.options_list:
			if voxelSetting_option == 'voxelBased':
				self.voxelSetting.options_vars[voxelSetting_option].set(0)
			else:
				self.voxelSetting.options_buttons[voxelSetting_option]['state'] = 'disabled'
				self.voxelSetting.options_vars[voxelSetting_option].set(0)
				self.voxelSetting.hide_entries(voxelSetting_option)

	def init_instruction(self, parent):
		font = 'Arial'; fontsize = 30
		x = 250; y = 70; width = 555; height = None
		
		label = tk.Label(parent)
		label['justify'] = 'left'
		label['text'] = 'Browse'
		label['bg'] = '#F0F0F0'
		label['fg'] = '#000000'
		label['anchor'] = 'nw'
		label['text'] = ''
		label['font'] = tkFont.Font(family=font,size=fontsize)
		label.place(x=x,y=y,width=width,height=height)

		return label

	def init_button(self,parent,params):
		button = tk.Button(parent)
		button['justify'] = 'center'
		button['text'] = params['text']
		button['relief'] = 'raised'
		button['bg'] = '#FFFFFF'
		button['fg'] = '#000000'
		button['state'] = 'normal'
		button['font'] = tkFont.Font(family=params['font'],size=params['fontsize'])
		button.place(x=params['x'],y=params['y'],width=params['width'],height=params['height'])
		button['command'] = params['command']

		return button

	def hide_buttons(self):
		self.browse_dir_button.place(x=-1000)
		self.next_button.place(x=-1000)
		self.back_button.place(x=-1000)
		self.load_button.place(x=-1000)
		self.save_button.place(x=-1000)
		self.logs.place(x=-1000)

	def show_button(self,button_key):
		params = self.params[button_key]

		if params['text'] == 'Next':
			self.next_button.place(x=params['x'])
		elif params['text'] == 'Back':
			self.back_button.place(x=params['x'])
		elif params['text'] == 'Browse':
			self.browse_dir_button.place(x=params['x'])
		elif params['text'] == 'Save':
			self.save_button.place(x=params['x'])
		elif params['text'] == 'Load':
			self.load_button.place(x=params['x'])
		elif params['text'] == 'logs':
			self.logs.place(x=params['x'])

	def init_path_entry(self, parent):
		
		params = self.params['path_entry']

		entry = tk.Entry(parent)
		entry.contents = StringVar()
		entry.contents.set('')
		entry['textvariable'] = entry.contents
		entry['borderwidth'] = '2px'
		entry['justify'] = 'left'
		entry['bg']= '#FFFFFF'
		entry['fg'] = '#333333'
		entry['font'] = tkFont.Font(family=params['font'],size=params['fontsize'])
		entry.place(x=params['x'],y=params['y'],width=params['width'],height=params['height'])

		return entry

	def show_path_entry(self):
		params = self.params['path_entry']
		self.path_entry.place(x=params['x']) # ,y=params['y'],width=params['width'],height=params['height'])

	def hide_path_entry(self):
		self.path_entry.place(x=-1000) #,y=-1000)

	def init_save_load_label(self,parent):
		params = self.params['save_load_label']

		label = tk.Label(parent)
		label['fg'] = '#000000'
		label['bg'] = '#F0F0F0'
		label['justify'] = 'left'
		label['text'] = ''
		label['anchor'] = 'nw'
		label['font'] = tkFont.Font(family=params['font'],size=params['fontsize'])
		label.place(x=params['x'],y=params['y'],width=params['width'],height=params['height'])

		return label

	def hide_save_load_label(self):
		self.save_load_label.place(x=-1000)
		pass

	def show_save_load_label(self):
		params = self.params['save_load_label']
		self.save_load_label['text'] = self.save_load_label_text
		self.save_load_label.place(x=params['x'])

	def show_step1(self, parent, controller):
		self.instruction['text'] = 'Set Extraction Parameters:'
		self.hide_buttons()
		self.hide_path_entry()
		self.directory_radio.hide()
		self.setting.show()
		self.voxelSetting.show()
		self.featureClass.show()
		self.show_save_load_label()
		self.show_button('next_button')
		self.show_button('save_button')
		self.show_button('load_button')

	def show_step2(self, parent, controller):
		self.instruction['text'] = 'Select Data Directory:'
		self.hide_buttons()
		self.hide_save_load_label()
		self.setting.hide()
		self.featureClass.hide()
		self.voxelSetting.hide()
		self.all_files_list.hide()
		self.image_files_list.hide()
		self.roi_files_list.hide()
		self.show_path_entry()
		self.show_button('next_button')
		self.show_button('back_button')
		self.show_button('browse_dir_button')
		self.directory_radio.show()
		self.next_button['text'] = 'Next'

	def show_step3(self, parent, controller):
		self.instruction['text'] = 'Select the Images Folder:'
		self.hide_buttons()
		self.directory_radio.hide()
		self.show_button('next_button')
		self.show_button('back_button')
		self.show_button('browse_dir_button')

	def show_step4(self, parent, controller):
		self.instruction['text'] = 'Select the ROIs Folder:'
		self.hide_buttons()
		self.all_files_list.hide()
		self.image_files_list.hide()
		self.roi_files_list.hide()
		self.show_path_entry()
		self.show_button('next_button')
		self.show_button('back_button')
		self.show_button('browse_dir_button')
		self.next_button['text'] = 'Next'

	def show_step5(self, parent, controller):
		self.instruction['text'] = 'Verify Images and ROIs:'
		self.hide_buttons()
		self.hide_path_entry()
		self.directory_radio.hide()
		self.all_files_list.show()
		self.image_files_list.show()
		self.roi_files_list.show()
		self.show_button('back_button')
		self.show_button('next_button')
		self.next_button['text'] = 'Next'

	def show_extraction(self, parent, controller):
		self.instruction['text'] = 'Proceed to Extraction:'
		self.hide_buttons()		
		self.all_files_list.hide()
		self.image_files_list.hide()
		self.roi_files_list.hide()
		self.show_button('back_button')
		self.show_button('next_button')
		self.next_button['text'] = 'Proceed'
		# self.show_button('logs')

class RadioSelect:
	def __init__(self, parent, params, controller):

		self.params = params
		self.init_radioselect(parent,controller)

	def init_radioselect(self,parent,controller):
		params = self.params

		self.radio_var = IntVar()

		radio_same = tk.Radiobutton(parent, text=params['text1'], variable = self.radio_var,value=1,command=controller.directory_radio)
		radio_same["fg"] = "#333333"
		radio_same["bg"]= "#F0F0F0"
		radio_same["justify"] = "left"
		radio_same["anchor"] = "nw"
		radio_same["font"] = tkFont.Font(family=params['font'],size=params['label_fontsize'])
		radio_same.place(x=params['x'], y=params['label_y'], width=params['width'], height=params['label_height'])

		self.radio_same = radio_same

		self.radio_diff_x = params['x']+params['dx2']
		radio_diff = tk.Radiobutton(parent, text=params['text2'], variable = self.radio_var,value=2,command=controller.directory_radio)
		radio_diff["fg"] = "#333333"
		radio_diff["bg"] = "#F0F0F0"
		radio_diff["justify"] = "left"
		radio_diff["anchor"] = "nw"
		radio_diff["font"] = tkFont.Font(family=params['font'],size=params['label_fontsize'])
		radio_diff.place(x=self.radio_diff_x, y=params['label_y'], width=params['width'], height=params['label_height'])

		self.radio_same = radio_same
		self.radio_diff = radio_diff

	def show(self):
		params = self.params
		self.radio_same.place(x=params['x'])
		self.radio_diff.place(x=self.radio_diff_x)

	def hide(self):
		self.radio_same.place(x=-1000)
		self.radio_diff.place(x=-1000)

class ExtractionOptions:
	def __init__(self, parent, params, controller):

		self.params = params
		self.options_list = [a_tuple[0] for a_tuple in self.params['option_list']]
		self.options_dicts = [a_tuple[1] for a_tuple in self.params['option_list']]
		self.options_buttons = {}
		self.options_vars = {}
		self.options_entries = {}
		self.options_strings = {}
		self.init_options(parent,controller)

	def init_options(self,parent,controller):
		params = self.params

		label = tk.Label(parent)
		label["fg"] = "#000000"
		label["bg"]= "#F0F0F0"
		label["justify"] = "left"
		label["anchor"] = "nw"
		label["text"] = params['text']
		label["font"] = tkFont.Font(family=params['font'],size=params['lb_fontsize'])
		label.place(x=params['x'], y=params['y'], width=params['lb_width'], height=params['lb_height'])

		self.label = label

		for i, option in enumerate(self.options_list):
			option_dict = self.options_dicts[i]
			var = IntVar()
			y_i = params['btn_y'] + i*params['btn_dy']
			check_button=tk.Checkbutton(parent,text=option,variable=var)
			check_button["fg"] = "#333333"
			check_button["bg"] = "#F0F0F0"
			check_button["justify"] = "left"
			check_button["relief"] = "flat"
			check_button["offvalue"] = "0"
			check_button["onvalue"] = "1"
			check_button["font"] = tkFont.Font(family=params['font'],size=params['btn_fontsize'])
			check_button.place(x=params['btn_x'],y=y_i)
			check_button.place(height=params['btn_height'])
			check_button['state'] = 'normal'
			check_button["command"] = controller.option_event
			
			self.options_buttons[option] = check_button
			self.options_vars[option] = var

			has_entry = option_dict['has_entry']
			has_entries = option_dict['has_entries']
			if has_entry:
				entry_string = StringVar() #HERER
				entry_string.set(option_dict['default'])
				entry = tk.Entry(parent,textvariable=entry_string)
				entry['font'] = tkFont.Font(family=params['font'],size=params['btn_fontsize'])
				entry.place(x=-1000,y=y_i,width=option_dict['width'])
				self.options_entries[option] = entry
				self.options_strings[option] = entry_string
			elif has_entries:
					option_entries = []
					option_strings = []
					for j in range(3):
						dx = option_dict['dx'][j]
						entry_string = StringVar()
						entry_string.set(option_dict['default'][j])
						entry = tk.Entry(parent,textvariable=entry_string)
						entry['font'] = tkFont.Font(family=params['font'],size=params['btn_fontsize'])
						entry.place(x=-1000,y=y_i+option_dict['dy'],width=option_dict['width'])
						option_entries.append(entry)
						option_strings.append(entry_string)
					self.options_entries[option] = option_entries
					self.options_strings[option] = option_strings

	def hide(self):
		self.label.place(x=-1000)
		for i, option in enumerate(self.options_list):
			self.options_buttons[option].place(x=-1000)
			option_dict = self.options_dicts[i]
			has_entry = option_dict['has_entry']
			has_entries = option_dict['has_entries']
			if has_entry:
				self.options_entries[option].place(x=-1000)
			elif has_entries:
					for j in range(3):
						self.options_entries[option][j].place(x=-1000)

	def show(self):
		params = self.params
		self.label.place(x=params['x'])

		for i, option in enumerate(self.options_list):
			self.options_buttons[option].place(x=params['btn_x'])
			if self.options_vars[option].get() == 1:
				option_dict = self.options_dicts[i]
				has_entry = option_dict['has_entry']
				has_entries = option_dict['has_entries']

				if has_entry:
					self.options_entries[option].place(x=params['x']+option_dict['dx'])
				elif has_entries:
					for j in range(3):
						self.options_entries[option][j].place(x=params['x']+option_dict['dx'][j])

	def show_entries(self, option):
		if self.options_vars[option].get() == 1:
			params = self.params
			option_index = self.options_list.index(option)
			option_dict = self.options_dicts[option_index]
			
			if option_dict['has_entry']:
					self.options_entries[option].place(x=params['x']+option_dict['dx'])
			elif option_dict['has_entries']:
				for j in range(3):
					self.options_entries[option][j].place(x=params['x']+option_dict['dx'][j])

	def hide_entries(self, option):
		option_index = self.options_list.index(option)
		option_dict = self.options_dicts[option_index]

		if option_dict['has_entry']:
			self.options_entries[option].place(x=-1000)
		elif option_dict['has_entries']:
			for j in range(3):
				self.options_entries[option][j].place(x=-1000)

	def deactivate_all(self):
		for option in self.options_list:
			self.options_buttons[option]['state'] = 'disabled'

	def activate_all(self):
		for option in self.options_list:
			self.options_buttons[option]['state'] = 'normal'

class FileList:
	def __init__(self, parent, params, controller):

		self.params = params
		self.init_filelist(parent,controller)

	def init_filelist(self,parent,controller):
		params = self.params

		label=tk.Label(parent)
		label["fg"] = "#333333"
		label["bg"]= "#F0F0F0"
		label["justify"] = "left"
		label["anchor"] = "nw"
		label["text"] = params['text']
		label["font"] = tkFont.Font(family=params['font'],size=params['label_fontsize'])
		label.place(x=params['x'], y=params['label_y'], width=params['width'], height=params['label_height'])


		listbox=tk.Listbox(parent)
		listbox["fg"] = "#333333"
		listbox["bg"]= "#FFFFFF"
		listbox["justify"] = "left"
		listbox["font"] = tkFont.Font(family=params['font'],size=params['fontsize'])
		listbox.place(x=params['x'], y=params['y'], width=params['width'], height=params['height'])

		if params['text']=='All Files':
			listbox.bind("<<ListboxSelect>>", controller.all_files_list_event)
		elif params['text']=='Image Files':
			listbox.bind("<<ListboxSelect>>", controller.image_files_list_event)
		elif params['text']=='ROI Files':
			listbox.bind("<<ListboxSelect>>", controller.roi_files_list_event)

		if params['button_font'] is not None:
			add_button=tk.Button(parent)
			add_button["fg"] = "#000000"
			add_button["bg"] = "#ffffff"
			add_button["justify"] = "center"
			add_button["text"] = 'Add'
			add_button["font"] = tkFont.Font(family=params['button_font'],size=params['button_fontsize'])
			add_button.place(x=params['add_x'],y=params['button_y'],width=params['button_width'],height=params['button_height'])
			add_button["state"] = 'normal'#"disabled"
			if params['text'] == 'Image Files':
				add_button['command'] = controller.image_add_button
			elif params['text'] == 'ROI Files':
				add_button['command'] = controller.roi_add_button

			del_button=tk.Button(parent)
			del_button["fg"] = "#000000"
			del_button["bg"] = "#ffffff"
			del_button["justify"] = "center"
			del_button["text"] = 'Remove'
			del_button["font"] = tkFont.Font(family=params['button_font'],size=params['button_fontsize'])
			del_button.place(x=params['del_x'],y=params['button_y'],width=params['button_width'],height=params['button_height'])
			del_button["state"] = "normal"
			if params['text'] == 'Image Files':
				del_button['command'] = controller.image_del_button
			elif params['text'] == 'ROI Files':
				del_button['command'] = controller.roi_del_button
		else:
			add_button = None
			del_button = None

		self.listbox = listbox
		self.label = label
		self.add_button = add_button
		self.del_button = del_button

	def hide(self):
		self.label.place(x=-1000)
		self.listbox.place(x=-1000)
		if self.params['button_font'] is not None:
			self.add_button.place(x=-1000)
			self.del_button.place(x=-1000)

	def show(self):
		params = self.params
		self.label.place(x=params['x'])
		self.listbox.place(x=params['x'],height=params['height'])
		if self.params['button_font'] is not None:
			self.del_button.place(x=params['del_x'],y=params['button_y'])
			self.add_button.place(x=params['add_x'],y=params['button_y'])

	def update_file_list(self,files_list):
		self.listbox.delete(0,self.listbox.size())
		for i, file in enumerate(files_list):
			self.listbox.insert( i, file)

class WizList:
	def __init__(self, parent):

		self.fontsize = 30
		self.padsize = 20
		self.dy = 90
		self.y = 70
		self.x = 50

		self.init_wizlist(parent)

	def init_wizlist(self,parent):

		textvars = ['Step 1','Step 2','Step 3','Step 4','Step 5']
		self.wiz_dict = {}

		i = 0
		for step_name in textvars:
			key = step_name

			self.wiz_dict[key] = self.add_step(step_name+'/5',parent)
			self.wiz_dict[key].place(x=self.x,y=self.y + i*self.dy)
			i+=1

	def add_step(self,text,parent):

		step=tk.Label(parent)

		step['fg'] = '#F0F0F0'
		step['bg'] = '#F0F0F0'
		step['justify'] = 'center'
		step['anchor'] = 'nw'
		step['text'] = text
		step['font'] = tkFont.Font(family='Arial',size=self.fontsize)

		return step

	def activate(self,key):
		self.wiz_dict[key]['fg'] = '#000000'
		pass

	def deactivate(self,key):
		self.wiz_dict[key]['fg'] = '#AAAAAA'
		pass

